/*
    Copyright (C) 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ECHARTPIE_H
#define ECHARTPIE_H

#include "echartbase.h"
#include "echarts.h"

/**
	* @brief Class to draw pie charts.
	* @ingroup Charts
	*
	* Just add one data set per piece (with one x-value). The x-Values
	* should be the values used to calculate the pie or call addPie().
	* If the x-values are already percents, call also setPercents(true).
	* Otherwise EChartPie calculates the percentages automatically.
	*
	* To set the number format and precision, use setNumberFormat() and
	* setNumberPrecision().
	*
	*/
class E_CHARTS_EXPORT EChartPie : public EChartBase
{
	Q_OBJECT
	Q_PROPERTY(bool percents READ percents WRITE setPercents)
	public:
		EChartPie(QWidget *parent = NULL);
		void setPercents(bool percent);
		inline bool percents() const { return arePercents; }

		void addPie(double value, const QString label, QColor color = QColor());

	protected:
		/**
			* @todo Calculate percentages once and not everytime.
			*/
		void drawChart(QPainter *painter, QRect viewPort);

	private:
		QString percentLabel(float percentage);

		bool arePercents;
		double PI;
	
};

#endif // ECHARTPIE_H

/**
* @example echarts.cpp
*
* @brief This example demonstrates the usage of EChartPie and EChartLine to draw pie and line charts.
*
* The example creates a main window with an central widget, which contains both charts.
* The charts can be exported by the Export action within there context menu.
*
* @image html example-echarts-screenshot.png
*
* The main window is a class with name MainWindow, defined in file mainwindow.h as follows:
* @snippet examples/echarts/mainwindow.h MainWindow
*
* The header contains only two custom functions. The first creates the pie chart widget and the second
* the line chart widget. Also a few private pointer are defined, to access the chart widgets and the central widget.
* The implementation is done within mainwindow.cpp.
*
* The first thing done in the constructor is the change of the window title. Afterwards a new central widget is created,
* a layout added and populated to the main window by calling setCentralWidget(). Then addChartPie() and addChartLine() creates
* the chart widgets.
*
* @snippet examples/echarts/mainwindow.cpp Constructor
*
* The pie chart widget is created and added to the layout of the widget, after the special LibEngsas slots from EWidgets are connected
* (like error, warning and information signals) by calling connectByPass().
*
* After settings the chart title with EChartBase::setChartTitle(), the input and output is configured. EChartPie::setPercent() is set
* to true, because the input values are already percent values. The precision is set to two digits, which is used to determine the number
* of shown digits in the chart later on. The values are added afterwards. For pie charts, every data set consists of an EChartValues object,
* which only contains one x-value. Therefore the y value is set to 0.
*
* @snippet examples/echarts/mainwindow.cpp addChartPie
*
* The first lines adding the line chart, are similar to the ones of the pie chart. Since the line chart has an x and y-axis,
* those needs to be configured. This is done by setting the tick values for both axis and their title. The date of the measured market
* shared is displayed on the x-axis in format MM.YY. On the y-axis, the market share in percent is displayed. Since the most common browser
* reaches 51 percent, the highest tick is the 50.
*
* Within an EChartLine, each line is represanted by one EChartValues object. Therefore an EChartValues object is created and value pairs
* for one browser (one line in the chart) are added. Each pair consists of a date as x-value and the market share at that time as y-value.
* After adding those pairs, the object is added to the chart by calling EChartBase::addDataSet(). The line is labeld with the specified string.
*
* @snippet examples/echarts/mainwindow.cpp addChartLine
*
* echarts.cpp contains the main() function to execute QApplication and load the MainWindow.
* 
* @example echartpie.qml
*
* This is an EChartPie widget included within an QML QtQuick application. If libengsas is installed into /usr/local you must
* include an import path with -I as follows
* @code
* qmlviewer -I /usr/local/lib echartpie.qml
* @endcode
* 
* If compiled with Qt5, QtQuick 2.0 is used and you must use qmlscene to show the qml scene:
* @code
* qmlscene -I /usr/local/lib echartpie.qml
* @endcode
*/
