/*
		Copyright (C) 2010 - 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EDIALOG_H
#define EDIALOG_H

#include "eglobal.h"
#include <QDialog>

#include "eobject.h"

class EPushButton;

class QMenu;
class QAction;
class QToolBar;
class QDockWidget;

/**
	* @brief Mostly the same as EWidget but for QDialog as base class.
	*
	* The paragraph below is not working, because the QMetaObject represents the base class.
	* Not the subclass.
	* It provides all needed standard functions. It sets the window title correctly and
	* restores and saves the geometry if the objectName() is not empty. To achieve this,
	* it sets the objectName() to ClassName with a lowered first character
	* (e.g.: MyEDialogSubClass gets the objectName() myEDialogSubclass) in the constructor.
	* Therefore you can override it in your constructor without any problems,
	* but in most cases the default name is enough.
	*
	* If you use the EHelpCenter also, you may want to setup a help button in the dialog.
	* You can do this by calling setHelp(). This connects the button with appropriate signals
	* and also sets the name and icon of the button.
	*
	* @see EWidget
	* @see EObject
	*/
class E_CORE_EXPORT EDialog : public QDialog
{
	Q_OBJECT
	public:
		/**
			* @brief Creates a EDialog instance with objectName() set to @p objName.
			*
			* This is the constructor you normaly want to use, to make sure loading and
			* saving geometry is working.
			*/
		EDialog ( QString objName, QWidget *parent = 0, Qt::WindowFlags flags = 0 );
		EDialog ( QWidget *parent = 0, Qt::WindowFlags flags = 0 );
		inline virtual ~EDialog(){ close(); }
		/**
			* @brief Sets the window title and prepends the QApplication::applicationName().
			*
			*/
    void setWindowTitle ( const QString &title );
		/**
			* @brief Set's the objectname and calls loadGeometry() afterwards.
			*
			* If setObjectName is called before all GUI-Elements are created
			* (like setupUi does), the loadGeometry() call does nothing.
			* Therefore you should call loadGeometry() yourself after all
			* gui elements are created.
			*/
		void setObjectName ( const QString &name );
		/**
			* @brief Can be called to restore geometry of this object.
			*
			* This function is called by setObjectName() after
			* setting the object name.
			* If the object name is empty, this function does nothing.
			*/
		void loadGeometry();
		/**
			* @brief Connects all standard signals.
			* @see EObject::connectByPass(QObject*)
			* @see EWidget::connectByPass(QObject*)
			*/
		void connectByPass(QObject *object){ EObject::connectByPass(this, object); }

  public slots:
		/**
			* @brief Used to reload settings the settings.
			*
			* This function is connected to doReloadSettings() and should be used
			* to actually do the reload of the settings. To indicate, that the settings
			* should be reloaded, emit reloadSettingsRequest().
			* Use @p error to describe the error for the user. And don't forget to
			* call the base implementation or emit doReload() yourself.
			*
			* @see doReloadSettings()
			* @see reloadSettingsRequest()
			* @see EWidget::reloadSettings()
			* @see EObject::reloadSettings()
			*/
		virtual inline void reloadSettings(){ emit doReloadSettings(); }

	protected slots:
		/**
			* @see error() if a child emits this signal, this class emits error()
			* (it passes the childs signal upwards)
			* @see EWidget::errorByPass()
			* @see EObject::errorByPass()
			*/
		virtual inline void errorByPass(const QString &message){ emit error(message); }
		/**
			* @see warning() passes the emitted signals from it's child forward
			* @see EWidget::warningByPass()
			* @see EObject::warningByPass()
			*/
		virtual inline void warningByPass(const QString &message){ emit warning(message); }
		/**
			* @see information() passes the emitted signals from it's child forward
			* @see EWidget::informationByPass()
			* @see EObject::informationByPass()
			*/
		virtual inline void informationByPass(const QString &message){ emit information(message); }
		/**
			* Passes the reloadEnableActions() signals from it's childs forward.
			*/
		virtual inline void reloadEnableActionsByPass(){ emit reloadEnableActions(); }
		/**
			* @see addMenu() passes the emitted signals from it's child forward
			* @see EWidget::addMenuByPass()
			* @see EObject::addMenuByPass()
			*/
		virtual inline void addMenuByPass(QMenu* newMenu, const QString &parent, QString before = QString()){ emit addMenu(newMenu, parent, before); }
		/**
			* @see addAction() passes the emitted signals from it's child forward
			* @see EWidget::addActionByPass()
			* @see EObject::addActionByPass()
			*/
		virtual inline void addActionByPass(QAction* newAction, const QString &parent, QString before = QString()){ emit addAction(newAction, parent, before); }
		/**
			* @see addToolBar() passes the emitted signals from it's child forward
			* @see EWidget::addToolBarByPass()
			* @see EObject::addToolBarByPass()
			*/
		virtual inline void addToolBarByPass(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea){ emit addToolBar(newToolBar, area); }
		/**
			* @see addNewDockWidget() passes the emitted signals from it's child forward
			* @see EWidget::addNewDockWidgetByPass()
			* @see EObject::addNewDockWidgetByPass()
			*/
		virtual inline void addNewDockWidgetByPass(QDockWidget *widget, const Qt::DockWidgetArea &area){ emit addNewDockWidget(widget, area); }
		/**
			* @see setCentralWidget() passes the emitted signals from it's child forward
			* @see EWidget::setCentralWidgetbyPass()
			* @see EObject::setCentralWidgetbyPass()
			*/
		virtual inline void setCentralWidgetByPass(QWidget *centralWidget){ emit setCentralWidget(centralWidget); }
		/**
			* @see showSettings() passes the emitted signals from it's child forward
			* @see EWidget::showSettingsByPass()
			* @see EObject::showSettingsByPass()
			*/
		virtual inline void showSettingsByPass(){ emit showSettings(); }
		/**
			* @see showHelp() passes the emitted signals from it's child forward
			* @see EWidget::showHelpByPass()
			* @see EObject::showHelpByPass()
			*/
		virtual inline void showHelpByPass(const QString &page){ emit showHelp(page); }
		/**
			* @see showStatusWidget() passes the emitted signals from it's child forward
			* @see EWidget::showStatusWidgetbyPass()
			* @see EObject::showStatusWidgetbyPass()
			*/
		virtual inline void showStatusWidgetByPass(QWidget* widget){ emit showStatusWidget(widget); }
		/**
			* @see reloadSettingsRequest() passes the emitted signals from it's child forward.
			* @see reloadSettings()
			* @see reloadSettingsByPass()
			* @see doReloadSettings()
			*/
		virtual inline void reloadSettingsRequestByPass(){ emit reloadSettingsRequest(); }
		/**
			* @brief Call this function to setup @p helpButton as a help button.
			*
			* Can be called by any subclass, to connect signal showHelp(QString) to helpButton.
			* It also set's the right icon, text and shortcut to the button
			*/
		void setHelp(QString page, EPushButton *helpButton = 0);
		
  protected:
		QString helpPage();
		void closeEvent(QCloseEvent *event);

	private slots:
		inline void help(){ emit showHelp(helpPage()); }
		
  private:
		void init();

		QString myHelpPage;
		
	signals:
		/**
			* The signals error(), warning() and information() are used to handle
			* user information. error() should be used in critical situations.
			* warning() is intendend to inform the user about not critical,
			* but important facts. information() are only to inform the user,
			* but they have no bad results.
			* To let the main application decide, how they are displayed,
			* only signals are emitted.
			* This signal is used to notify about an error with @p message
			* @see warning()
			* @see information()
			* @see errorByPass()
			* @see EWidget::error()
			* @see EObject::error()
			*/
		void error(const QString &message);
		/**
			* Used to notify about an warning with @p message.
			* @see error()
			* @see information()
			* @see warningByPass()
			* @see EWidget::warning()
			* @see EObject::warning()
			*/
		void warning(const QString &message);
		/**
			* Used to notify about an information with @p message.
			* @see error()
			* @see warning()
			* @see informationByPass()
			* @see EWidget::information()
			* @see EObject::information()
			*/
		void information(const QString &message);
		/**
			* @brief Request reloading of enabled actions in EMainWindow.
			*/
		void reloadEnableActions();
		/**
			* Allows to add menu @p newMenu to the main window.
			* It is added to an existing menu with QObject::objectName() @p parent
			* before the action or menu with object name @p before. If @p before is empty,
			* it is appended at the bottom.
			* @see addAction() to add an action
			* @see addToolBar()
			* @see addMenuByPass()
			* @see EWidget::addMenu()
			* @see EObject::addMenu()
			*/
		void addMenu(QMenu* newMenu, const QString &parent, QString before = QString());
		/**
			* Adds the action @p newAction to the main window.
			* It is added to an existing menu with QObject::objectName() @p parent
			* before the action or menu with object name @p before. If @p before is empty,
			* it is appended at the bottom.
			* @see addMenu()
			* @see addToolBar()
			* @see addActionByPass()
			* @see EWidget::addAction()
			* @see EObject::addAction()
			*/
		void addAction(QAction* newAction, const QString &parent, QString before = QString());
		/**
			* Used to add the toolbar @p newToolBar to the main window.
			* @see addMenu()
			* @see addAction()
			* @see addToolBarByPass()
			* @see EWidget::addToolBar()
			* @see EObject::addToolBar()
			*/
		void addToolBar(QToolBar* newToolBar, Qt::ToolBarArea area = Qt::TopToolBarArea);
		/**
			* Used to add the dock widget @p widget to the main window at the given @p area.
			* The main window automatically adds an checkabe show and hide action to the view menu
			* with the title of the @p widget.
			* @see setCentralWidget()
			* @see addNewDockWidgetByPass()
			* @see EWidget::addNewDockWidget()
			* @see EObject::addNewDockWidget()
			*/
		void addNewDockWidget(QDockWidget *widget, const Qt::DockWidgetArea &area);
		/**
			* Sets @p centralWidget as the new central widget.
			* @attention old central widget is destroyed
			* @see addNewDockWidget()
			* @see setCentralWidget()
			* @see EWidget::setCentralWidget()
			* @see EObject::setCentralWidget()
			*/
		void setCentralWidget(QWidget *centralWidget);
		/**
			* request to show the settings dialog.
			* @see showSettingsByPass()
			* @see EWidget::showSettings()
			* @see EObject::showSettings()
			*/
		void showSettings();
		/**
			* help should be shown and @p page should be opened.
			* @see showHelpByPass()
			* @see EWidget::showHelp()
			* @see EObject::showHelp()
			*/
		void showHelp(const QString &page);
		/**
			* Adds @p widget to the status bar as a permanent widget.
			* @see showStatusWidgetByPass()
			* @see EWidget::showStatusWidget()
			* @see EObject::showStatusWidget()
			*/
		void showStatusWidget(QWidget* widget);
		/**
			* @brief Indicates, that a reload of the settings is needed.
			*
			* Therefore this signal is used to bring the reload message upwards to the main window
			* or whatever.
			* @see reloadSettingsRequestByPass()
			* @see doReloadSettings()
			* @see reloadSettings()
			* @see EWidget::reloadSettingsRequest()
			* @see EObject::reloadSettingsRequest()
			*/
		void reloadSettingsRequest();
		/**
			* @brief Emitted to bring the reload request back to the farest child.
			* @see reloadSettings()
			* @see reloadSettingsRequest()
			* @see EWidget::doReloadSettings()
			* @see EObject::doReloadSettings()
			*/
		void doReloadSettings();
};

#endif // EDIALOG_H
