/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef EPART_H
#define EPART_H

#include "eglobal.h"
#include "eobject.h"

#include "ehelpcenter.h"
#include "eversion.h"

class ESettingsModelItem;
class EDependencyItem;
class ECmdLineArgs;

#include <QImage>
#include <QUrl>

class QFile;
class QMenu;
class QAction;
class QToolBar;
class QDockWidget;

/**
	* @brief Adds a libEngsasVersion() function to your EPart.
	* 
	* Just insert this macro into your EPart header to avoid programming
	* the libEngsasVersion() function.
	*/ 
#define EPART_LIBENGSAS_VERSION \
public: \
	inline QString libEngsasVersion() const { return QString(ENGSAS_VERSION_STRING); }; \
private:

/**
	* @brief Implement this class to create an object, which can be used in several applications.
	*
	* It is compareable to KDE KParts, but also a bit differently in detail.
	* The loading of an EPart contains the following steps:
	* @li Create an instance of the wanted part
	*     (Normally you wnat to use ELibrary::loadEPart() for this step)
	* @li connect the signals with EObject::connectByPass()
	*     (Do not forget to connect showSetupWizard() also.)
	* @li call init() on your loaded part
	* @li If you want to show the starting widgets of the EPart, call show().
	*
	* To implement libEngsasVersion() function, just add EPART_LIBENGSAS_VERSION to your header.
	* 
	* @note Do not forget to register the functions icon(), shortName() and description()
	*    as an Q_PROPERTY like shown below:
	*    @code
	*    Q_PROPERTY(QString name READ shortName)
	*    Q_PROPERTY(QString description READ description)
	*    Q_PROPERTY(QImage icon READ icon)
	*    @endcode
	* 	You can use the class TestEParts from @ref libengsas-testing to make sure, that your parts pass some basic
	*   quality checks.
	*
	* @code
	* QString error;
	* part = ELibrary::loadEPart("Partname", error, false);
	* if(!part || !error.isEmpty()){
	*   emit error(tr("Could not load Partname!\n(%1)").arg(error));
	*   return EPart::Failed;
	* }
	* connectByPass(part);
	* connect(part, SIGNAL(showSetupWizard()), this, SLOT(showSetupWizardByPass()));
	* part->init(args, false);
	* @endcode
	*
	* To use the EPart as a standalone application, nothing needs to be done, but
	* installing the EPart into the ELibrary EPart-directory. Afterwards just call
	* @code
	* epartexe Partname
	* @endcode
	* to open the EPart as a standalone application.
	*
	* @see EPartMainWindow to integrate your EPart into an standalone application
  *
  * @todo Plugins should be allowed to be licensed under different licenses
  *       than GPLv2, e.g. LGPL for not soo free sources
  *       -> this will lead to provide a license by every epart and will also
  *          need some changes in epartexe
  * @todo Add function to epart interface providing a help text regarding the
  *       eparts general functionality and usage (special descriptions regarding
  *       the options are located in the option list)
	*/
class E_CORE_EXPORT EPart : public EObject {
	public:
		/**
			* @brief Used to identify the result of the init() function.
			*/
		enum InitState
		{
			Exit, 			/**< means, that the init was successful, but the application should exit now.
										* This can be the case, if commandline arguments are used, which requires only
										* some processing and the processing was done successful without any further
										* interaction neeed. */
			Failed, 		/**< init or the started operations have failed */
			Successful	/**< everything is alright */
		};

		/**
			* @brief Should return the directory, where all translations for this part can be found.
			*
			* The translation directory should contain all translation files provided by this EPart (not Qt or LibEngsas ones).
			* @return A QString, pointing to the directory which contains the translation files.
			*    Return an empty string, if you do not provide translations.
			* @see resourceDir()
			*/
		virtual QString translationDir() const = 0;
		/**
			* @brief Should return the resource directory, where all resources for this part can be found.
			*
			* The resource directory should contain all resource files provided by this EPart (not Qt or LibEngsas ones).
			* @return A QString, pointing to the directory which contains the resource (*.rcc) files.
			*    Return an empty string, if you do not provide resources.
			* @see translationDir()
			*/
		virtual QString resourceDir() const = 0;
		/**
			* @brief Return a list a supported command line options.
			*
			* Some like -v or -h are added by the epartexe automatically.
			*/
		virtual EngSaS::CmdLineOptions cmdLineArgs() const = 0;
		/**
			* @brief Called to setup the EPart.
			*
			* This is the first function called from an EPart, after creating
			* an instance of the part. You should not emit showSetupWizard()
			* within this context, because the setupwizard is normally shown later
			* on, to let the application load all EParts and show one setupwizard
			* for all EParts.
			*
			* The specified command line arguments are given
			* with @p args and can be parsed with help of ECmdLineArgs. @p standalone is set
			* to true, if the EPart is the only part, which is loaded. The Epart may organize it's
			* menus and actions in another way then. For example in standalone mode, it's actions
			* are placed under File-menu and if @p standalone is false, the actions are placed in an
			* submenu below the File-menu.
			* @return the result state of the operations
			* @note Before you call this function you should connect nearly all of the signals
			*/
		virtual InitState init(ECmdLineArgs *args, bool standalone = false) = 0;
		/**
			* @brief Return the real long name of the EPart.
			*
			* @see shortName()
			*/
		virtual QString name() const = 0;
		/**
			* @brief Return the short name of the EPart (used as window title and for QSettings).
			*
			* @see name()
			*/
		virtual QString shortName() const = 0;
		/**
			* @brief Return an icon to represent the EPartMainWindow.
			*
			*/
		virtual QImage icon() const = 0;
		/**
			* @brief Return the help description of the EPart.
			*/
    virtual QString helpDescription() const = 0;
    /**
     * @brief Return the description of the EPart.
     */
		virtual QString description() const = 0;
		/**
			* @brief The Url to the website of this EPart.
			*
			* Should be relative to engsas.de to get localized absolute url in EAboutDialog.
			*/
		virtual QUrl website() const = 0;
		/**
			* @brief Return the version of this EPart.
			*/
		virtual QString versionString() const = 0;
		/**
			* @brief The LibEngsas version at compile time of your EPart.
			* 
			* To implement this function at your EPart, just add
			* EPART_LIBENGSAS_VERSION to your header file.
			* 
			* @see EPART_LIBENGSAS_VERSION
			*/
		virtual QString libEngsasVersion() const = 0;
		/**
			* @brief Used to setup the EHelpCenter correctly for this part.
			*
			* @see EHelpCenter
			* @see EHelpCenterSetup
			*/
		virtual EHelpCenterSetup helpSetup() = 0;
		/**
			* @brief Return all needed dependencies (like used libraries) as or in an EDependencyItem.
			*
			*/
		virtual EDependencyItem *dependencies() = 0;
		/**
			* @brief Return a list with ESetupObjects, which should be placed in an ESetupWizard before show() is called.
			*/
		virtual EngSaS::SetupWizardObjects setupWizardObjects() = 0;
		/**
			* @brief A list of persons, who should be mentioned at the about dialog.
			*
			* @code
			* EngSaS::AboutPersons result;
			* result << new EAboutPerson("Important", "Very", "very.important@example.org", EAboutPersons::Author);
			* result << new EAboutPerson("Important", "Lesser", EAboutPersons::Thanks);
			* return result;
			* @endcode
			*/
		virtual EngSaS::AboutPersons aboutPersons() = 0;
		/**
			* @brief The returned item should contain all needed settings for this EPart.
			*
			* @see ESettingsModelItem
			* @see ESettingsDialog
			*/
		virtual ESettingsModelItem *settingsModelItem() = 0;

  public slots:
		/**
			* @brief This should show the standard widget of this module.
			*/
		virtual void show() = 0;

	signals:
		virtual void showSetupWizard() = 0;
};

#if QT_VERSION >= 0x050000
#define LibEngsasEPartIID "de.engsas.libengsas.EPart"
Q_DECLARE_INTERFACE(EPart, LibEngsasEPartIID)
#else
Q_DECLARE_INTERFACE ( EPart, "LibEngsas.EPart/0.1.1" );
#endif

#endif // EPART_H
