/*
    Copyright (C) 2013 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ESPREADSHEET_H
#define ESPREADSHEET_H

#include "eglobal.h"
#include "eobject.h"

#include "espreadsheetcell.h"

class ESpreadsheetProvider;

#include <QModelIndex>
#include <QFont>
#include <QBrush>

class QAbstractItemModel;

/**
	* @brief A spreadsheet, within a spreadsheet file.
	* @ingroup Spreadsheets
	*
	* A spreadsheet consists of cells of type ESpreadsheetCell, which are ordered in rows and columns.
	* An object of this type can be used to manipulate the hold data (see e. g. setData(ESpreadsheetCell)).
	* To retrieve the cell content use data(int, int).
	*
	* ESpreadsheet can also deal with QAbstractItemModel. It provides the functions setData(QAbstractItemModel*, QModelIndex)
	* and data(QAbstractItemModel*, QModelIndex) to write and read data from and into a QAbstractItemModel.
	*
	* To append an spreadsheet to a file (and getting an pointer to an ESpreadsheet object back),
	* use ESpreadsheetHandler::appendSpreadsheet().
	*
	* To implement your own ESpreadsheet subclass within your ESpreadsheetProvider,
	* implement setData(ESpreadsheetCell), data(int, int), rowCount() and columnCount(). May be you also want to overload
	* setName().
	*
	* @see ESpreadsheetCell
	* @see ESpreadsheetHandler
	* @see ESpreadsheetProvider
	*/
class E_CORE_EXPORT ESpreadsheet : public EObject
{
	Q_OBJECT
	public:
		ESpreadsheet(ESpreadsheetProvider* provider, QObject* parent = NULL);
		
		/**
			* @brief Used by setData(QAbstractItemModel*, QModelIndex, Header) to determine which header to add.
			*/
		enum Header {
			HeaderNone = 0x0,
			HeaderVertical = 0x1,
			HeaderHorizontal = 0x2,
			HeaderAll = HeaderVertical | HeaderHorizontal
		};
		Q_DECLARE_FLAGS(Headers, Header)

		QString name() const;
		virtual void setName(QString name);
		ESpreadsheetProvider *provider() const;
		void setProvider(ESpreadsheetProvider *provider);

		/**
			* @brief Adds the data of the childs of index @p root to this spreadsheet.
			*
			* To also add the header of the model to the spreadsheet, set @p header appropriatly.
			* 
			* Internally it calls setData(QVariant, QFont, Qt::AlignmentFlag, QBrush, QBrush).
			*
			* @see setData(ESpreadsheetCell)
			* @see setForeground()
			* @see setBackground()
			* @see setFont()
			* @see setAlignment()
			* @see data(int, int)
			* @see data(QAbstractItemModel*, QModelIndex)
			*/
		bool setData(QAbstractItemModel* model, ESpreadsheet::Headers header = ESpreadsheet::HeaderNone, QModelIndex root = QModelIndex());
		/**
			* @brief Adds @p value to this spreadsheet at ESpreadsheetCell::row() and ESpreadsheetCell::column of @p cell.
			* 
			* To manipulate the text font, provide @p font, @p alignment and @p foreground. To specifiy a background color,
			* set @p background.
			*
			* @see setData(QAbstractItemModel*, QModelIndex)
			* @see setForeground()
			* @see setBackground()
			* @see setFont()
			* @see setAlignment()
			* @see data(int, int)
			* @see data(QAbstractItemModel*, QModelIndex)
			* 
			* @note This function should be implemented by the plugin providing an ESpreadsheetProvider.
			*/
		virtual bool setData(ESpreadsheetCell cell) = 0;

		/**
			* @brief Sets the foreground of cell @p row and @p column to @p foreground.
			*
			* @see setData(ESpreadsheetCell)
			* @see setData(QAbstractItemModel*, QModelIndex)
			* @see setBackground()
			* @see setFont()
			* @see setAlignment()
			*/
		bool setForeground(int row, int column, QBrush foreground);
		/**
			* @brief Sets the background of cell @p row and @p column to @p background.
			*
			* @see setData(ESpreadsheetCell)
			* @see setData(QAbstractItemModel*, QModelIndex)
			* @see setForeground()
			* @see setFont()
			* @see setAlignment()
			*/
		bool setBackground(int row, int column, QBrush background);
		/**
			* @brief Sets the font of cell @p row and @p column to @p font.
			*
			* @see setData(ESpreadsheetCell)
			* @see setData(QAbstractItemModel*, QModelIndex)
			* @see setForeground()
			* @see setBackground()
			* @see setAlignment()
			*/
		bool setFont(int row, int column, QFont font);
		/**
			* @brief Sets the alignment of cell @p row and @p column to @p alignment.
			*
			* @see setData(ESpreadsheetCell)
			* @see setData(QAbstractItemModel*, QModelIndex)
			* @see setForeground()
			* @see setBackground()
			* @see setFont()
			*/
		bool setAlignment(int row, int column, Qt::AlignmentFlag alignment);

		/**
			* @brief Loads the data of this spreadsheet into @p model as childs of index @p root.
			* 
			* Internally it uses data(int row, int column);
			*
			*/
		bool data(QAbstractItemModel *model, QModelIndex root = QModelIndex()) const;
		/**
			* @brief Returns the data of the cell @p row, @p column.
			* 
			* @note This function should be implemented by the plugin providing an ESpreadsheetProvider.
			*/
		virtual ESpreadsheetCell data(int row, int column) const = 0;

		/**
			* @brief Returns the number of rows within this spreadsheet.
			* 
			* @note This function should be implemented by the plugin providing an ESpreadsheetProvider.
			*/
		virtual int rowCount() const = 0;
		/**
			* @brief Returns the number of columns within this spreadsheet.
			* 
			* @note This function should be implemented by the plugin providing an ESpreadsheetProvider.
			*/
		virtual int columnCount() const = 0;

	protected:
		/**
			* @brief Constructs an ESpreadsheet without a provider.
			*
			* You should set a provider with setProvider() within the constructor of your subclass!
			*/
		ESpreadsheet(QObject* parent = NULL);

	private:
		QString myName;
		ESpreadsheetProvider *myProvider;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(ESpreadsheet::Headers)

#endif // ESPREADSHEET_H
