/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ESQLINIT_H
#define ESQLINIT_H

#include "eglobal.h"
#include "eobject.h"

#include "esqltable.h"
// #include "esqlfield.h"
#include "esqldatabase.h"

/**
	* @defgroup SQL
	*
	* This module contains all classes related to SQL databases.
	*/

/**
	* @brief Class to manage database structures.
	* @ingroup SQL
	*
  * This class is able to create and modify complexe database structures.
	* You can define a structure in scheme() and this class handles the
	* creation of this structure or updates an existing one automatically.
	* To allow more customization, you can register update routines, 
	* to get a more customized post processsing of
	* updates.
	*
	* @note This class doesn't delete any table automatically. This is for
	*   data security at the database. Normally data moving must be done
	*   before deleting a table. You can do this with help of the mentioned
	*   update routines and also delete the table within this routine.
	*
	* To use this class, subclass it and implement scheme(). There you should
	* setup the needed meta data by calling setup(). Afterwards, you can
	* use addTable() to add tables. At last you can use addUpdateRoutine()
	* to add some custom post update processing.
	*
	* Here is an example scheme function:
	* @code
	* void Example:scheme() {
	*   setup("example", "Example Human Name", 9);
	*   // add a table with standard id field
	*   // the table will be called "example_users",
	*   // because the name set by setup is "example".
  *   ESqlTable *users = addTable ( "users" );
	*   // add a second field
  *   field = users->appendField ( "name", ESqlField::Varchar, true );
  *   field->setLength ( 45 );
  *   users->addIndex ( field );
	*   // ad a theird field
	*   field = users->appendField ( "givenName", ESqlField::Varchar );
	*   field->setLength ( 45 );
	*   // add an update routine from version < 6 to version = 6:
	*   addUpdateRoutine(6, &Example::updateFrom5To6);
	* }
	* @endcode
	*
	* @see ESqlTable
	* @see ESqlField
  * @todo error message when inserting of vaules have failed
  * @todo check needed rights and ask for username and password with extended rights, if they are needed
  *       for example things like create tables, ...
  */
class E_CORE_EXPORT ESqlInit : public EObject
{
	Q_OBJECT
	public:
		/**
			* possible SQL permissions
			* CREATE assumes ALTER and CREATE VIEW
			*/
		enum privileges { SELECT, INSERT, UPDATE, DEL, CREATE, ALTER };
		enum Status { BadSettings, LoggedIn, LoginFailed, UpdateNeeded, UpdateFailed, NoUpdateNeeded };

		ESqlInit ( QObject *parent = 0,
							QString username = QString(), QString host = QString(),
							QString database = QString(), QString driver = QString(),
							QString password = QString());
		virtual ~ESqlInit();
		/**
			* @brief Logs in and after logging it's check the structure of the database against the given one at the scheme() function
			*
			* For convenience, this function enables GUI automatically, if QApplication is available. If only QCoreApplication is available,
			* GUI is disabled automatically.
			*
			* @param runUpdate is set to false login returns true if an update is needed
			* @param runUpdate is set to true, login runs the update and return true if the updates are done successfull
		  */
		ESqlInit::Status login(bool loginOnly = true, bool runUpdate = false);
		ESqlInit::Status loginGuiless(bool loginOnly = true, bool runUpdate = false);

		/**
			* @brief Returns the label set with setup().
			*
			* This string is used as a prefix for all tables of this module.
			*/
		QString label() const;
		/**
			* @brief Returns the schema version set with setup().
			*
			*/
		int schemaVersion() const;
		/**
			* @brief Returns the database name, currently managed.
			*/
		QString databaseName() const;
		/**
			* @brief Emits the signal information() with content @p msg.
			*
			* Use this function to add information messages to the log
			* in your updateRoutines.
			*/
		void addInformation(QString msg);

	public slots:
		void reloadSettings();

  protected:
    /**
			* @brief Stores the database scheme.
			*
      */
    virtual void scheme() = 0;
		/**
			* @brief Registers the update routine.
			*
			* @code
			* static bool Example::updateFrom3To4 (ESqlInit *caller, QList< ESqlTable* > tables, QString& errorString )
      * {
			*   Q_UNUSED(tables);
			*   caller->addInformation(tr("Your log message"));
			*   // 1. delete useless table userhasmaterials
			*   QSqlQuery query;
			*   if(!query.exec(QString("DROP TABLE %1_userhasmaterials").arg(caller->label())){
			*     errorString.append ( query.lastError().text() );
			*     return false;
			*   }
			*   return true;
			* }
			* 
			* void Example::scheme(){
			*   addUpdateRoutine(4, &Example::updateFrom3To4);
			* }
			* @endcode
			*
			* In the update routine, you can access the label set with setup() with help of caller->label().
			* To get the database name, which is used by current connection, use caller->databaseName().
			* If you want to add a message to the logs, use caller->addInformation().
			*
			* @param toVersion Version which is reached, after the updateroutine is run
			* @note The updateroutine @p functionPointer should handle a QString& to give back error messages
			* @note and it get's a pointer to the calling ESqlInit object.
			* @see label()
			* @see databaseName()
			* @see addInformation()
			*/
		void addUpdateRoutine(int toVersion, bool (*functionPointer)(ESqlInit *, QList<ESqlTable*>, QString&) );

		/**
			* @brief Adds a new table to the list of tables.
			*
			* The added table will be named @p name with prefix of the
			* name set by setup() (if @p addPrefix is true) and gets an id field, if
			* @p appendFieldId is true (Default).
			* @return Pointer to the ESqlTable object, which represents
			*    the newly added table .
			*/
    ESqlTable *addTable ( QString name, bool appendFieldId = true, bool addPrefix = true );
		/**
			* @brief Adds a view to the list of views.
			*
			* The added view will be named @p name with prefix of the name
			* set by setup() (if @p addPrefix is true). The view itself is
			* defined with help of a select statement, which is the second
			* parameter @p selectStatement.
			*/
		void addView( QString name, QString selectStatement, bool addPrefix = true );
		/**
			* @brief Setup the needed meta data of this module.
			*
			* The chosen @p name is used as a prefix for some tables
			* and also an entry in the dbmodulesversion is done with
			* this @p name. This entry contains also @p label and @p version.
			*/
		void setup(QString name, QString label, int version);

  private slots:
    /**
      * initializes the update
      */
    bool updateDb();

  private:
		/**
			* The internal login function
			*/
		ESqlInit::Status login(bool loginOnly, bool runUpdate, bool enableGui);
    /**
      * checks wether update of the structure is needed or not
      * it also checks wether structure is newer than needed
      * which might be bad too
      */
    bool updateNeeded();
    /**
      * does the update
      */
    bool doUpdate();
		/**
      * compares to tables
      * in mode @p create = false it returns true if both
      * tables are identical and false if not
      * if @p update is set to true, it updates the table or even create it.
			* @note Fields are not deleted here, to give you the chance to do something with the
			*   data, which is in the deleted fields. Therefore, you must delete fields in an
			*   updateRoutine. Also default values are not touched, if fields are deleted to avoid
			*   errors (because fields are not deleted, but default values needs the new structure).
      */
    bool compareTables ( ESqlTable *table, QString &errorString, bool create = false );
    /**
      * checks, if view with current statement exists
      * @param view the view which a create statement should be created
      * @param error a pointer to a QString where the error message is appended
      * @param create create view if it not exist or with other select statement
      * @return true if view already exists and is the same as needed
			* @todo Because, checking is quite complicated, the views are dropped and created again, this should be improved!
      */
    bool compareViews ( QStringList view, QString &errorString, bool create = false );
    /**
      * create views statement
      * @param view index of view at QList views;
      * @return SQL query to create view
      */
    QString createViewStatement ( QStringList view );
		
		/**
			* @brief Runs all needed and registered update routines.
			*
			* It runs all registered update routines, which are needed to reach
			* @p toVersion from @p fromVersion. If an error occurs, @p errorString 
			* is filled and false is returned.
			*
			* To register an update routine, call addUpdateRoutine().
			* @note All tables are updated automatically, but some operations,
			*   like convert data to a new structure, can not be done automatically.
			*   In this case, you should register an updaet routine.
			*/
		bool runUpdateRoutines(int fromVersion, QString& errorString, int toVersion);

		/**
      * @brief Initializes the structure by calling scheme().
			*
			* It allso adds the standard table "dbmoduleversions" as first table.
      */
    void init();
		
    QString dbName, versionTable;
		bool transactions, gui;
    EngSaS::SQL::DatabaseType dbType;
    QList<ESqlInit::privileges> neededPrivs;
		int actVersion;
		QString username, password, host, driver;
		ESqlDatabase db;
		/**
      * current version of db structure
      */
    int currentVersion;
    /**
      * prefix of all tables which belongs to this module
      */
    QString thisName;
    QString moduleLabel;
    /**
      * a list of all tables of the database which are needed by this module
      */
    QList<ESqlTable*> tables;
    QList<QStringList> views;
		/**
			* the first int (QMap-Key) is the result version, which is reached, if updateFunction runs successfully. 
			* updateFunction should than return true
			* @see DBInit for examples
			*/
		QMap<int, bool (*)(ESqlInit *, QList<ESqlTable*>, QString&)> updateRoutines;

  signals:
		/**
			* @brief Set the text above the upper progress bar to @p msg.
			*/
    void actualStep ( QString msg );
		/**
			* @brief Increases the value of the progress bar by 1.
			*/
    void finishedStep();
		/**
			* @brief Sets the maximum of the progress bar to @p num.
			*/
    void neededSteps ( int num );
};

#endif // ESQLINIT_H
