/*
		Copyright (C) 2013 - 2015 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

		This library is free software; you can redistribute it and/or
		modify it under the terms of the GNU Lesser General Public
		License as published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option) any later version.

		This library is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

		You should have received a copy of the GNU Lesser General Public
		License along with this library; if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
		MA 02110-1301  USA.
*/

#ifndef ESQLLOGIN_H
#define ESQLLOGIN_H

#include "eobject.h"
#include "eglobal.h"

class ESqlLoginDialog;

// only as long, ESqlLogin::exec() is in use, to allow
// old code to compare exec() result against enum from QDialog
#include <QDialog>

namespace KWallet {
	class Wallet;
}

/**
	* @brief Class to login into a database.
	* @ingroup SQL
	*
	* If GUI is not disabled, it requests the login data
	* with help of ESqlLoginWidget, if needed. Otherwise
	* only the specified login data is used.
	*
	* Usage:
	* @code
	* ESqlLogin login("Database connection for Super Application", "", true, this);
	* if(login.loadAndCheckSettings()){
	*   if(login.login() != ESqlLogin::Result){
	*     // do some error handling here
	*   }
	*   else{
	*     // do something with database
	*   }
	* }
	* @endcode
	*
	* @note It is used by ESqlInit and some other classes. Since the use of ESqlInit, EDBSettings
	*   and ESetupWizardDB is recommended, this class should only be used, if you need a connection
	*   with a different user than used within the mentioned class before.
	*
	* @see ESqlInit
	* @see ESqlLoginWidget
	*/
class E_CORE_EXPORT ESqlLogin : public EObject
{
	Q_OBJECT
	Q_ENUMS(Result)
	public:
		enum Result { Failed, LoggedIn, Canceled };

		Q_DECL_DEPRECATED ESqlLogin(QObject *parent = NULL);
		ESqlLogin(bool enableGui, QObject *parent = NULL);
		ESqlLogin (QString title, QString newConnName, bool enableGui = true, QObject *parent = NULL);

		QStringList data();
		void reloadSettings();
		/**
			* @brief Loads all settings and checks them.
			*/
		bool loadAndCheckSettings(QString username = QString(),
															QString host = QString(),
															QString database = QString(),
															QString driver = QString(),
															QString password = QString());
		/**
			* @brief Checks the database driver and database name combination.
			*/
		static bool checkDBSettings();
		/**
			* @brief Marked for removel, use checkDBSettings() instead, which does exactly the same.
			*/
		Q_DECL_DEPRECATED static bool checkSettings() { return checkDBSettings(); }

	public slots:
		/**
			* @brief Tries to login into the database.
			*
			* If needed and enabledGui is true, it tries to get the login data with
			* ESqlLoginWidget, if needed.
			*/
		ESqlLogin::Result login();
		/**
			* @brief Marked for removel, use login() instead, which does exactly the same, but allows better result distinguishing.
			*
			*/
		Q_DECL_DEPRECATED int exec();

	private slots:
		void showSettingsDialog();

	private:
		void init(bool enableGui, QString connectionName = QString());
		ESqlLogin::Result doLogin();
		ESqlLoginDialog *createDialog() const;
		bool checkDBSettingsPrivate();

		QString host, driver, database;
		QString connName, username, password;
		QString dialogTitle;
		bool gui, withoutDatabase, useSSL;
		QPointer<KWallet::Wallet> wallet;
};

#endif // ESQLLOGIN_H
