/*
    Copyright (C) 2010 - 2012 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301  USA.
*/

#ifndef ESQLTABLE_H
#define ESQLTABLE_H

#include "eglobal.h"

#include "esqlfield.h"

#include <QString>
#include <QStringList>
#include <QMultiHash>

/**
	* @brief Represents an sql table.
	* @ingroup SQL
	*
	* This class is used with ESqlInit to define database schemas.
	*
	* @see ESqlInit
	* @see ESqlTable
	*/
class E_CORE_EXPORT ESqlTable : public EObject
{
	Q_OBJECT
  public:
		/**
			* @brief Actions, which are done, if an foreign key is changed.
			*/
    enum FkAction {
			NoAction, /**< Nothing is done */
			SetNull, 	/**< The fk-field is set to NULL */
			Cascade, 	/**< See MySQL Handbook */
			Restrict 	/**< See MySQL Handbook */
		};

    ESqlTable ( QString name = "" );
    ~ESqlTable();
    bool isValid();
    QString name();
    void setName ( QString name );
    QList<ESqlField*> fields();
    void appendField ( ESqlField *append );
	/**
	  * @brief Adds a new field (also called column) to the table.
	  *
	  * To add fields of type ESqlField::Varchar, you may want to use appendFieldVarchar().
	  */
    ESqlField *appendField ( QString name = "", ESqlField::Type type = ESqlField::NoType, bool notNull = false, bool autoIncrement = false );
	/**
		* append's the id standard field
		* name: id
		* type: unsigned integer
		* autoincremented, not null
		*/
	void appendFieldId (QString name = "id");
	/**
	  * @brief Appends a field of type ESqlField::Varchar with lenght @p length.
	  *
	  * It's a shorter form for
	  * @code
	  * ESqlField *field = results->appendField("fieldName", ESqlField::Varchar, notNull, false);
	  * field->setLength(45);
	  * @endcode
	  */
	ESqlField * appendFieldVarchar ( QString name = "", uint length = 45, bool notNull = false );
	
    bool appendValues ( QStringList value );
    QList<QStringList> values();
		/**
			* @brief Changes column @p fieldName from this table to reference another table.
			*
			* The added column will have the name @p fieldName and references
			* the field @p refFieldName at the table @p refTable. With @p del
			* and @p update, you can specify actions on deletion and update
			* of the referenced field.
			* @see addForeignKey(ESqlField*, ESqlTable*, ESqlTable::FkAction, ESqlTable::FkAction)
			* @see addForeignKey(QString, QString, ESqlTable::FkAction, ESqlTable::FkAction, QString)
			*/
    void addForeignKey ( QString fieldName,
                         QString refTable,
                         QString refFieldName = "id",
                         ESqlTable::FkAction del = ESqlTable::NoAction,
                         ESqlTable::FkAction update = ESqlTable::NoAction );
		/**
			* @brief Changes @p ourField to reference another table.
			*
			* The field @p ourField at this table is changed to reference 
			* the standard id field of the table @p refTable. With @p del
			* and @p update, you can specify actions on deletion and update
			* of the referenced field.
			* @see addForeignKey(QString, QString, QString, ESqlTable::FkAction, ESqlTable::FkAction)
			* @see addForeignKey(QString, QString, ESqlTable::FkAction, ESqlTable::FkAction, QString)
			*/ 
		void addForeignKey ( ESqlField *ourField, ESqlTable *refTable,
                         ESqlTable::FkAction del = ESqlTable::NoAction,
                         ESqlTable::FkAction update = ESqlTable::NoAction );
		/**
			* @brief Adds renamed field from @p refTable to reference that table.
			*
			* Tries to add a copy of the id field of @p refTable to this table.
			* It renames the field to "refTableNameId" or if given to
			* @p newFieldName. Also an foreignkey entry is added for the newly
			* added field. With @p del and @p update, you can specify actions
			* on deletion and update of the referenced field. With @p idFieldName,
			* the name of the used field name in @p refTable can be changed from
			* the default value into another value.
			* @note the part ModuleName_ from tablename is stripped out, cds_cds becomes cdsId and NOT cds_cdsId
			* @see appendFieldId()
			*/
		void addForeignKey ( ESqlTable *refTable, 
												 QString newFieldName = QString(),
												 ESqlTable::FkAction del = ESqlTable::NoAction,
                         ESqlTable::FkAction update = ESqlTable::NoAction,
												 QString idFieldName = "id" );
    QString sqlConstraintsString ( EngSaS::SQL::DatabaseType type = EngSaS::SQL::Other );
		/**
			* overlaods addIndex(field length, order), to use it without ESqlFields
			*/
    void addIndex ( QString fieldName, unsigned int length = 0, Qt::SortOrder order = Qt::AscendingOrder );
		/**
			* adds an Index to field @p field
			* The Index gets the length @p length, if length is greater null.
			* The used sortorder is @p order
			*/
		void addIndex ( ESqlField *field, unsigned int length = 0, Qt::SortOrder order = Qt::AscendingOrder );
    QString indizes();
		
		/**
			* @brief Creates the SQL statements which are needed to create this table.
			*
			* With @p dbName a database can be given, in which the table should be created.
			* If @p sqlite is set to true, it respects sqlite specific behaviour.
			* @return SQL query string which can be used to create the table.
			*/
		QString createStatement(QString dbName, EngSaS::SQL::DatabaseType type);

  private:
    QString tableName;
    QList<ESqlField*> myFields;
    QList<QStringList> myValues;
    QList<QStringList> constraints;
    QStringList myIndizes;
    QMultiHash<ESqlTable::FkAction, QString> fkActionMap;
};

#endif // ESQLTABLE_H
